@@ -1,13 +1,18 @@
 package Image::Info::BMP;
-$VERSION = '1.01';
+$VERSION = '1.03';
 use strict;
 
-sub process_file{
+use constant _CAN_LITTLE_ENDIAN_PACK => $] >= 5.009002;
+
+sub process_file {
     my($info, $source, $opts) = @_;
     my(@comments, @warnings, @header, %info, $buf, $total);
 
     read($source, $buf, 54) == 54 or die "Can't reread BMP header: $!";
-    @header = unpack("SLS2L2V2S2L2V2L2", $buf);
+    @header = unpack((_CAN_LITTLE_ENDIAN_PACK
+		      ? "vVv2V2Vl<v2V2V2V2"
+		      : "vVv2V2V2v2V2V2V2"
+		     ), $buf);
     $total += length($buf);
 
     if( $header[9] && $header[9] < 24 ){
@@ -23,7 +28,11 @@ sub process_file{
     else{
 	$info->push_info(0, "file_ext" => "bmp"); # || dib
     }
-    $info->push_info(0, "height", abs($header[7]));
+
+    $info->push_info(0, "height", (_CAN_LITTLE_ENDIAN_PACK
+				   ? abs($header[7])
+				   : ($header[7] >= 2**31 ? 2**32 - $header[7] : $header[7])
+				  ));
     $info->push_info(0, "resolution", "$header[12]/$header[13]");
     $info->push_info(0, "width", $header[6]);
     $info->push_info(0, "BitsPerSample" => $header[9]);
@@ -46,7 +55,7 @@ sub process_file{
     if( $header[5] > 40 ){
 	read($source, $buf, $header[5]-40);  # XXX test
 	$total += length($buf);
-	my @v5 = unpack("L38", $buf);
+	my @v5 = unpack("V38", $buf);
 	splice(@v5, 5, 27);
 	$info->push_info(0, "BMP_MaskRed", $v5[0]);
 	$info->push_info(0, "BMP_MaskGreen", $v5[1]);
@@ -88,29 +97,12 @@ sub process_file{
 }
 1;
 __END__
-warn if height is negative and compress is not RGB or BITFILEDS (0 or 3)
-ICO and CUR support?
-### v5
-If bit depth is 0, it relies upon underlying JPG/PNG :-(
-Extra Information
-    DWORD        bV5RedMask; 
-    DWORD        bV5GreenMask; 
-    DWORD        bV5BlueMask; 
-    DWORD        bV5AlphaMask; 
-    DWORD        bV5CSType; 
-    CIEXYZTRIPLE bV5EndPoints; #3*CIEXYZ #CIEXYZ = 3*FXPT2DOT30#FXPT2DOT30 = long
-    DWORD        bV5GammaRed; 
-    DWORD        bV5GammaGreen; 
-    DWORD        bV5GammaBlue; 
-    DWORD        bV5Intent; 
-    DWORD        bV5ProfileData; 
-    DWORD        bV5ProfileSize; 
 
 =pod
 
 =head1 NAME
 
-Image::Info:BMP - Windows Device Indepdent Bitmap support for Image::Info
+Image::Info::BMP - Windows Device Indepdent Bitmap support for Image::Info
 
 =head1 SYNOPSIS
 
@@ -156,15 +148,41 @@ The number of color I<available> is 2 ^ B<BitsPerSample>.
 
 =back
 
+=head1 METHODS
+
+=head2 process_file()
+    
+	$info->process_file($source, $options);
+
+Processes one file and sets the found info fields in the C<$info> object.
+
 =head1 SEE ALSO
 
 L<Image::Info>
 
 =head1 NOTES
 
-For more information about BMP see:
+For more information about BMP see L<http://msdn.microsoft.com>.
 
- http://msdn.microsoft.com
+Random notes:
+
+  warn if height is negative and compress is not RGB or BITFILEDS (0 or 3)
+  ICO and CUR support?
+  ### v5
+  If bit depth is 0, it relies upon underlying JPG/PNG :-(
+  Extra Information
+    DWORD        bV5RedMask; 
+    DWORD        bV5GreenMask; 
+    DWORD        bV5BlueMask; 
+    DWORD        bV5AlphaMask; 
+    DWORD        bV5CSType; 
+    CIEXYZTRIPLE bV5EndPoints; #3*CIEXYZ #CIEXYZ = 3*FXPT2DOT30#FXPT2DOT30 = long
+    DWORD        bV5GammaRed; 
+    DWORD        bV5GammaGreen; 
+    DWORD        bV5GammaBlue; 
+    DWORD        bV5Intent; 
+    DWORD        bV5ProfileData; 
+    DWORD        bV5ProfileSize; 
 
 =head1 DIAGNOSTICS
 
@@ -190,8 +208,6 @@ Jerrad Pierce <belg4mit@mit.edu>/<webmaster@pthbb.org>
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
-=cut
-
 =begin register
 
 MAGIC: /^BM/
