@@ -3,7 +3,7 @@
 #
 # Author: Slaven Rezic
 #
-# Copyright (C) 2009 Slaven Rezic. All rights reserved.
+# Copyright (C) 2009,2011 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -12,7 +12,7 @@ package Image::Info::SVG;
 
 use strict;
 use vars qw($VERSION @PREFER_MODULE $USING_MODULE);
-$VERSION = '2.00';
+$VERSION = '2.02';
 
 @PREFER_MODULE = qw(Image::Info::SVG::XMLLibXMLReader
 		    Image::Info::SVG::XMLSimple
@@ -144,7 +144,7 @@ modify it under the same terms as Perl itself.
 
 =begin register
 
-MAGIC: /^<\?xml/
+MAGIC: /^(<\?xml|[\012\015\t ]*<svg\b)/
 
 Provides a plethora of attributes and metadata of an SVG vector grafic.
 
